#!/usr/bin/env ruby
#####################################################################################
# Copyright 2016 Kenneth Evensen <kenneth.evensen@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
######################################################################################
#
# Contact Info: <kenneth.evensen@redhat.com> and <lester@redhat.com>
#
######################################################################################

load 'client/OcpClient.rb'

class Base

  attr_reader :response

  def initialize(version, api, path, namespace)
    @client = OcpClient.instance
    @api = api
    @version = version
    @path = path
    @namespace = namespace
    @response = nil

    @endpoint = "/"+@api+"/"+@version

    if !@namespace.nil?
      @endpoint = @endpoint+"/namespaces/"+@namespace+"/"+@path
    elsif !@path.nil?
      @endpoint = @endpoint+"/"+@path
    end

  end

  def setup(url, noverifyssl, pretty, debug , token=nil, clientcertfile=nil, clientkeyfile=nil, clientcafile=nil)
    @client.setup(url, noverifyssl, pretty, debug , token, clientcertfile, clientkeyfile, clientcafile)
  end

  def setup_by_config_file(configfile, pretty=nil, debug=nil)
    @client.setup_by_config_file(configfile, pretty, debug)
  end

  def list
    @response = @client.get(@endpoint)
    return @response
  end

  def create(body)
    @response = @client.post(@endpoint,body)
    return @response
  end

  def update(body, name)
    @response = @client.put(@endpoint+"/"+name,body)
    return @response
  end

  def delete(name)
    @response = @client.delete(@endpoint+"/"+name)
    return @response
  end

  private
  def is_a_number?(s)
    s.to_s.match(/\A[+-]?\d+?(\.\d+)?\Z/) == nil ? false : true
  end
end
