#!/usr/bin/env ruby
#####################################################################################
# Copyright 2015 Kenneth Evensen <kenneth.evensen@redhat.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
#
#####################################################################################
#
# Contact Info: <kenneth.evensen@redhat.com> and <lester@redhat.com>
#
#####################################################################################

load 'base/v1/core/ObjectReference.rb'
load 'base/v1/core/ObjectMeta.rb'

class RoleBinding < OcpApi

  def initialize(namespace=nil)
    unless namespace.nil?
      super("rolebindings",namespace)
      @users = Array.new
      @subjects = Array.new
      @namespace = namespace
    else
      super("rolebindings")
    end
  end

  def create_role_binding(user, role)
    data = create(create_body(user, role))
    return data
  end

  def update_role_binding(user, role)
    data = update(create_body(user, role), role)
    return data
  end


  def to_s
    data = nil
    data = list
    return data
  end

  private

  def create_body(user, role)
    @users.push(user)
    subj = ObjectReference.new
    subj.kind = "User"
    subj.name = user

    @subjects << subj.get_hash

    roleRef = ObjectReference.new
    roleRef.name = role

    objMeta = ObjectMeta.new
    objMeta.name = role
    objMeta.namespace = @namespace
    objMeta.resourceversion = "1001"

    body = {
            'metadata' => objMeta.get_hash,
            'groupNames' => nil,
            'userNames' => @users,
            'subjects' => @subjects,
            'roleRef' => roleRef.get_hash
          }
          return body
  end

end
